.object-tools {
  display: flex;
}
.ow-object-notify {
  align-items: center;
}

/* Object Notification */

.ow-object-notify:active,
.ow-object-notify:focus {
  outline: 0 none;
}
#ow-silence-label {
  line-height: 20px;
  margin: auto;
  text-transform: uppercase;
  letter-spacing: 0.5px;
}
.ow-object-notification-option-container {
  width: 200px;
  margin-top: 5px;
}
#ow-notification-help-text {
  padding: 10px 10px;
  margin: 0;
  color: #828282;
  background: rgba(255, 255, 255, 0.5);
}
#ow-object-notification-loader .loader {
  height: 3em;
  width: 3em;
  margin: 10px auto;
}

/* Icons */
.ow-object-notify-bell {
  mask-image: url('../../openwisp-notifications/images/icons/icon-bell.svg');
  -webkit-mask-image: url('../../openwisp-notifications/images/icons/icon-bell.svg');
}
.ow-object-notify-slash-bell {
  mask-image: url('../../openwisp-notifications/images/icons/icon-bell-slash.svg');
  -webkit-mask-image: url('../../openwisp-notifications/images/icons/icon-bell-slash.svg');
}
#ow-object-notify .ow-icon {
  display: inline-block;
  width: 15px;
  height: 14px;
  position: relative;
  bottom: -2px;
  background: #fff;
  -webkit-mask-size: 15px;
  -webkit-mask-repeat: no-repeat;
  mask-size: 15px;
  mask-repeat: no-repeat;
  margin-right: 2px;
}
#ow-object-notify span{
  pointer-events: none;
}
@media screen and (max-width: 768px) {
  .object-tools {
    justify-content: space-between;
    overflow: visible !important;
  }
  .ow-object-notify {
    margin: auto auto auto 0px;
  }
}
